/* Copyright (c) Michal Altair Valasek - Altairis, 2006 | www.altairis.cz
 * 
 * This program is free software licensed under terms of the BSD license:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name of Michal Altair Valasek, Altairis nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace Papouch.Thermometers {

    /// <summary>
    /// This class represents the "TME" ethernet thermometer from PaPouch Electronic, connected using TCP/IP.
    /// </summary>
    public class TcpSensor : ISensor {
        private const string TcpDataFormat = @"^\*B1E1[\+\-]\d{3}\.\d{1}$"; // expected reply format for TCP sensor
        string host;                                                        // host name or IP address
        int port;                                                           // TCP port to connect (default 10001)
        int readTimeout = 15000;                                            // TCP read timeout in miliseconds (default 15 s)

        /// <summary>
        /// Gets or sets read timeout in miliseconds. Default is 15000.
        /// </summary>
        public int ReadTimeout {
            get { return readTimeout; }
            set { readTimeout = value; }
        }

        /// <summary>
        /// Gets or sets TCP port number. Default is 10001.
        /// </summary>
        public int Port {
            get { return port; }
            set {
                if (value < 1 || value > 65535) throw new ArgumentOutOfRangeException("value", value, "Port must be between 1 and 65535.");
                port = value;
            }
        }

        /// <summary>
        /// Gets or sets TME host name or IP address.
        /// </summary>
        public string Host {
            get { return host; }
            set {
                if (value == null) throw new ArgumentNullException();
                if (value == "") throw new ArgumentException("Host cannot be empty.");
                host = value;
            }
        }

        /// <summary>
        /// Initializes instance of this class using default port.
        /// </summary>
        /// <param name="host">TME host name or IP address.</param>
        public TcpSensor(string host) {
            this.Host = host;
            this.Port = 10001;
        }

        /// <summary>
        /// Initializes instance of this class.
        /// </summary>
        /// <param name="host">TME host name or IP address.</param>
        /// <param name="port">TCP port number.</param>
        public TcpSensor(string host, int port) {
            this.Host = host;
            this.Port = port;
        }

        // ISensor implementation
        float ISensor.ReadSample() {
            // Get raw data
            string td = null;
            System.Net.Sockets.TcpClient c = new System.Net.Sockets.TcpClient();
            try {
                c.ReceiveTimeout = this.ReadTimeout;
                c.Connect(this.Host, this.Port);
                td = new System.IO.StreamReader(c.GetStream()).ReadLine();
            }
            finally {
                c.Close();
            }

            // Parse data
            if (!Regex.IsMatch(td, TcpDataFormat)) throw new FormatException(string.Format("Sensor returned data in unexpected format: '{0}'.", td));
            return float.Parse(td.Substring(5), System.Globalization.NumberStyles.Float, System.Globalization.CultureInfo.InvariantCulture);
        }
    }

}
